@@ -30,7 +30,6 @@
 #include <gnome.h>
 #include <glib/gi18n.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <libgnomeui/gnome-window-icon.h>
 #include <gconf/gconf-client.h>
 #include <games-scores.h>
 #include <games-scores-dialog.h>
@@ -513,9 +512,11 @@ static void
 game_over (void)
 {
 	int pos;
+	GamesScoreValue hiscore;
 
 	set_statusbar_message (_("Game Over!"));
-	pos = games_scores_add_score (highscores, (GamesScoreValue)score);
+	hiscore.plain = score;
+	pos = games_scores_add_score (highscores, hiscore);
 	show_scores (pos, TRUE);
 	update_score_state ();
 	return;
@@ -1213,13 +1214,21 @@ game_about_callback (GtkAction *action, gpointer *data)
 			     "Szabolcs B\xc3\xa1n <shooby@gnome.hu>",
 			     NULL};
 
+   const gchar *documenters[] = {"Emese Kovács",
+				 NULL};
+
    gtk_show_about_dialog (GTK_WINDOW(app),
 			  "name", _("Five or More"),
 			  "version", VERSION,
 			  "comments", _("GNOME port of the once-popular Color Lines game"),
-			  "copyright", "Copyright \xc2\xa9 1997-2004 Free Software Foundation, Inc.",
+			  "copyright", "Copyright \xc2\xa9 1997-2006 Free Software Foundation, Inc.",
+			  "license", "GPL 2+",
 			  "authors", authors,
+			  "documenters", documenters,
 			  "translator_credits", _("translator-credits"),
+			  "logo-icon-name", "gnome-five-or-more",
+			  "website", "http://www.gnome.org/projects/gnome-games/",
+			  "wrap-license", TRUE,
 			  NULL);
    return TRUE;
 }       
@@ -1875,25 +1884,28 @@ main (int argc, char *argv [])
 	GtkWidget *preview_hbox;
 	GtkUIManager *ui_manager;
 	GnomeClient *client;
+	GnomeProgram *program;
 	int i;
-	
+
+	setgid_io_init ();
+
 	rgen = g_rand_new ();
 	
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	gnome_program_init ("glines", VERSION,
-			    LIBGNOMEUI_MODULE,
-			    argc, argv,
-			    GNOME_PARAM_POPT_TABLE, NULL,
-			    GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
+	program = gnome_program_init ("glines", VERSION,
+				      LIBGNOMEUI_MODULE,
+				      argc, argv,
+				      GNOME_PARAM_POPT_TABLE, NULL,
+				      GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
 
 	highscores = games_scores_new (&scoredesc);
 
         init_config (argc, argv);
 	games_stock_init ();
-	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/glines.png");
+	gtk_window_set_default_icon_name ("gnome-five-or-more");
 	client = gnome_master_client ();
 
 	g_signal_connect (G_OBJECT (client), "save_yourself",
@@ -2007,5 +2019,9 @@ main (int argc, char *argv [])
 
 	g_object_unref (conf_client);
 
+	gnome_accelerators_sync();
+
+	g_object_unref (program);
+
 	return (0);
 }
