@@ -1,3 +1,5 @@
+/* -*- mode:C; indent-tabs-mode:nill; tab-width:8; c-basic-offset:8 -*- */
+
 #ifndef __blockops_h__
 #define __blockops_h__
 
@@ -25,7 +27,8 @@
 
 enum SlotType 
 {
-	EMPTY, 
+	EMPTY,
+	TARGET,
 	FALLING, 
 	LAYING
 };
@@ -54,12 +57,22 @@ public:
 	void emptyField(int filled_lines, int fill_prob);
 	void putBlockInField(bool erase);
 	int getLinesToBottom();
+	bool isFieldEmpty (void);
+	void setUseTarget (bool);
+	bool getUseTarget () { return useTarget; };
 
 private:
+	void putBlockInField (int bx, int by, int blocknr, int rotation,
+			      SlotType fill);
 	bool blockOkHere(int x, int y, int b, int r);
 	void eliminateLine(int l);
 
+	bool useTarget;
+
 protected:
+	void clearTarget ();
+	void generateTarget ();
+
 	Block **field;
 
 	int blocknr;
