@@ -23,12 +23,9 @@
 
 #include <config.h>
 #include <gnome.h>
-#include <libgnomeui/gnome-window-icon.h>
 #include <gdk/gdkkeysyms.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <games-clock.h>
 
-#include <sys/time.h>
 #include <string.h>
 
 #include "gnothello.h"
@@ -43,7 +40,6 @@ GtkWidget *drawing_area;
 GtkWidget *tile_dialog;
 GtkWidget *black_score;
 GtkWidget *white_score;
-GtkWidget *time_display;
 
 GdkPixmap *buffer_pixmap = NULL;
 GdkPixmap *tiles_pixmap = NULL;
@@ -64,10 +60,7 @@ guint tiles_to_flip = 0;
 gint64 milliseconds_total = 0;
 gint64 milliseconds_current_start = 0;
 
-gint timer_valid = 0;
-
-gint bcount;
-gint wcount;
+guint game_in_progress;
 
 gint8 pixmaps[8][8] = {{0,0,0,0,0,0,0,0},
 		      {0,0,0,0,0,0,0,0},
@@ -87,11 +80,13 @@ gint8 board[8][8] = {{0,0,0,0,0,0,0,0},
 		    {0,0,0,0,0,0,0,0},
 		    {0,0,0,0,0,0,0,0}};
 
-MoveHistory game[61];
-
-gint8 move_count = 0;
+guint whose_turn;
+gint8 move_count;
+gint bcount;
+gint wcount;
 
 extern guint flip_final_id;
+extern gint8 squares[64];
 
 int session_flag = 0;
 int session_xpos = -1;
@@ -105,17 +100,20 @@ GdkGC *gridGC[2] = { 0 };
 
 static void new_network_game_cb(GtkWidget *widget, gpointer data);
 
-static const struct poptOption options[] = {
-  {NULL, 'x', POPT_ARG_INT, &session_xpos, 0, NULL, NULL},
-  {NULL, 'y', POPT_ARG_INT, &session_ypos, 0, NULL, NULL},
-  {"server", 's', POPT_ARG_STRING, &game_server, 0, N_("Iagno server to use")},
-  {NULL, '\0', 0, NULL, 0}
+static const GOptionEntry options[] = {
+  {"x", 'x', 0, G_OPTION_ARG_INT, &session_xpos, N_("X location of window"), 
+   N_("X")},
+  {"y", 'y', 0, G_OPTION_ARG_INT, &session_ypos, N_("Y location of window"), 
+   N_("Y")},
+  {"server", 's', 0, G_OPTION_ARG_STRING, &game_server, 
+   N_("Iagno server to use"), N_("NAME")},
+  {NULL}
 };
 
 GnomeUIInfo game_menu[] = {
         GNOMEUIINFO_MENU_NEW_GAME_ITEM(new_game_cb, NULL),
 
-	GNOMEUIINFO_ITEM(N_("New net_work game"), NULL, new_network_game_cb, NULL),
+	GNOMEUIINFO_ITEM(N_("New Net_work Game"), NULL, new_network_game_cb, NULL),
 
 	GNOMEUIINFO_SEPARATOR,
 
@@ -128,83 +126,8 @@ GnomeUIInfo game_menu[] = {
 	GNOMEUIINFO_END
 };
 
-/*
-GnomeUIInfo black_level_radio_list[] = {
-	{ GNOME_APP_UI_ITEM, N_("_Disabled"),
-	  N_("Disable the computer player"),
-	  black_level_cb, 0, NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
-
-	{ GNOME_APP_UI_ITEM, N_("Level _One"),
-	  N_("Enable the level 1 computer player"),
-	  black_level_cb, (gpointer) 1, NULL, GNOME_APP_PIXMAP_DATA, NULL, 0,
-	  0, NULL },
-
-	{ GNOME_APP_UI_ITEM, N_("Level _Two"),
-	  N_("Enable the level 2 computer player"),
-	  black_level_cb, (gpointer) 2, NULL, GNOME_APP_PIXMAP_DATA, NULL, 0,
-	  0, NULL },
-
-	{ GNOME_APP_UI_ITEM, N_("Level Th_ree"),
-	  N_("Enable the level 3 computer player"),
-	  black_level_cb, (gpointer) 3, NULL, GNOME_APP_PIXMAP_DATA, NULL, 0,
-	  0, NULL },
-
-	GNOMEUIINFO_END
-};
-
-GnomeUIInfo white_level_radio_list[] = {
-	{ GNOME_APP_UI_ITEM, N_("_Disabled"),
-	  N_("Disable the computer player"),
-	  white_level_cb, (gpointer) 0, NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
-
-	{ GNOME_APP_UI_ITEM, N_("Level _One"),
-	  N_("Enable the level 1 computer player"),
-	  white_level_cb, (gpointer) 1, NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
-
-	{ GNOME_APP_UI_ITEM, N_("Level _Two"),
-	  N_("Enable the level 2 computer player"),
-	  white_level_cb, (gpointer) 2, NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
-
-	{ GNOME_APP_UI_ITEM, N_("Level Th_ree"),
-	  N_("Enable the level 3 computer player"),
-	  white_level_cb, (gpointer) 3, NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
-	GNOMEUIINFO_END
-};
-
-GnomeUIInfo black_level_menu[] = {
-	GNOMEUIINFO_RADIOLIST(black_level_radio_list),
-	GNOMEUIINFO_END
-};
-
-GnomeUIInfo white_level_menu[] = {
-	GNOMEUIINFO_RADIOLIST(white_level_radio_list),
-	GNOMEUIINFO_END
-};
-
-GnomeUIInfo comp_menu[] = {
-};
-*/
-
-/*
-GnomeUIInfo settings_computer_submenu[] = {
-        GNOMEUIINFO_SUBTREE_HINT(N_("_Dark"),
-				 N_("Configure the dark computer player"),
-				 black_level_menu),
-        GNOMEUIINFO_SUBTREE_HINT(N_("_Light"), 
-				 N_("Configure the light computer player"),
-				 white_level_menu),
-
-	GNOMEUIINFO_SEPARATOR,
-
-	GNOMEUIINFO_TOGGLEITEM(N_("_Quick moves"),
-			       N_("Turn on quick computer moves"),
-			       quick_moves_cb, NULL),
-	GNOMEUIINFO_END
-};
-*/
-
 GnomeUIInfo settings_menu[] = {
-	GNOMEUIINFO_MENU_PREFERENCES_ITEM (properties_cb, NULL),
+        GNOMEUIINFO_MENU_PREFERENCES_ITEM (properties_cb, NULL),
         GNOMEUIINFO_END
 };
 
@@ -249,10 +172,9 @@ new_game_cb (GtkWidget *widget, gpointer data)
 void
 undo_move_cb (GtkWidget *widget, gpointer data)
 {
-	gint8 which_computer;
-	gint i, j;
+	gint8 which_computer, xy;
 
-	if ((black_computer_level && white_computer_level) || !move_count)
+	if ((black_computer_level && white_computer_level) || move_count == 4)
 		return;
 
 	if (flip_final_id) {
@@ -262,53 +184,27 @@ undo_move_cb (GtkWidget *widget, gpointer data)
 
 	game_in_progress = 1;
 
-	if (black_computer_level || white_computer_level) {
-		if (black_computer_level)
-			which_computer = BLACK_TURN;
-		else
-			which_computer = WHITE_TURN;
-		move_count--;
-		while (game[move_count].me == which_computer && move_count > 0) {
-			pixmaps[game[move_count].x][game[move_count].y] = 100;
-			move_count--;
-		}
-		pixmaps[game[move_count].x][game[move_count].y] = 100;
-		memcpy (board, game[move_count].board, sizeof (gint8) * 8 * 8);
-	} else {
-		move_count--;
-		memcpy (board, game[move_count].board, sizeof (gint8) * 8 * 8);
-		pixmaps[game[move_count].x][game[move_count].y] = 100;
+ 	which_computer = OTHER_PLAYER (whose_turn);
+ 	undo();
+ 	board_copy();
+ 	xy=squares[move_count];
+ 	pixmaps[xy%10-1][xy/10-1] = 100;
+	if ((((whose_turn == WHITE_TURN) && white_computer_level) ||
+	     ((whose_turn == BLACK_TURN) && black_computer_level))
+	    && (move_count > 4)) {
+ 		undo();
+ 		board_copy();
+ 		xy=squares[move_count];
+ 		pixmaps[xy%10-1][xy/10-1] = 100;
 	}
 
-	whose_turn = game[move_count].me;
-
 	if (whose_turn == WHITE_TURN)
 		gui_message (_("Light's move"));
 	else
 		gui_message (_("Dark's move"));
 
-	wcount = 0;
-	bcount = 0;
-
-	for (i = 0; i < 8; i++)
-		for (j = 0; j < 8; j++) {
-			if (board[i][j] == WHITE_TURN)
-				wcount++;
-			if (board[i][j] == BLACK_TURN)
-				bcount++;
-		}
-
 	gui_status ();
-
-	if (timer_valid) {
-		games_clock_stop (GAMES_CLOCK (time_display));
-		gtk_widget_set_sensitive (time_display, FALSE);
-		games_clock_set_seconds (GAMES_CLOCK (time_display), 0);
-		timer_valid = 0;
-	}
-
 	tiles_to_flip = 1;
-
 	check_computer_players ();
 }
 
@@ -324,14 +220,6 @@ black_level_cb (GtkWidget *widget, gpointer data)
 
         black_computer_level = tmp;
 
-        if (game_in_progress) {
-
-                games_clock_stop (GAMES_CLOCK (time_display));
-                gtk_widget_set_sensitive (time_display, FALSE);
-                games_clock_set_seconds (GAMES_CLOCK (time_display), 0);
-                timer_valid = 0;
-        }
-
         check_computer_players ();
 }
 
@@ -347,63 +235,29 @@ white_level_cb (GtkWidget *widget, gpointer data)
 
         white_computer_level = tmp;
 
-        if (game_in_progress) {
-                games_clock_stop (GAMES_CLOCK (time_display));
-                gtk_widget_set_sensitive (time_display, FALSE);
-                games_clock_set_seconds (GAMES_CLOCK (time_display), 0);
-                timer_valid = 0;
-        }
-
         check_computer_players ();
 }
 
 void
 about_cb (GtkWidget *widget, gpointer data)
 {
-	static GtkWidget *about;
-	GdkPixbuf *pixbuf = NULL;
 	const gchar *authors[] = {"Ian Peters", NULL};
-	const gchar *documenters[] = {
-	    			      NULL
-    	};
-    	const gchar *translator_credits = _("translator-credits");
-	
-	if (about != NULL) {
-                gtk_window_present (GTK_WINDOW (about));
-		return;
-	}
 
-	{
-		char *filename = NULL;
+	const gchar *documenters[] = {"Eric Baudais", NULL};
 
-		filename = gnome_program_locate_file 
-			(NULL,
-			 GNOME_FILE_DOMAIN_APP_PIXMAP,  ("iagno.png"),
-			 TRUE, NULL);
-		if (filename != NULL)
-		{
-			pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
-			g_free (filename);
-		}
-	}
-
-	about = gnome_about_new (_("Iagno"), VERSION, 
-				 "Copyright \xc2\xa9 1998-2004 Ian Peters",
-				 _("GNOME version of the popular Othello disk "
-				   "flipping game."), 
-				 (const char **)authors, 
-				 (const char **)documenters,
-				 strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
-				 pixbuf);
-	
-	if (pixbuf != NULL)
-		gdk_pixbuf_unref (pixbuf);
-				
-	g_signal_connect (G_OBJECT (about), "destroy",
-			  G_CALLBACK (gtk_widget_destroyed), &about);
-	gtk_window_set_transient_for (GTK_WINDOW(about), GTK_WINDOW(window));
-
-	gtk_widget_show (about);
+	gtk_show_about_dialog (GTK_WINDOW (window),
+			       "name", _("Iagno"),
+			       "version", VERSION,
+			       "copyright", "Copyright \xc2\xa9 1998-2006 Ian Peters",
+			       "license", "GPL 2+",
+			       "comments",_("A disk flipping game derived from Reversi."),
+			       "authors", authors,
+			       "documenters", documenters, 
+			       "translator_credits", _("translator-credits"),
+			       "logo-icon-name", "gnome-iagno",
+			       "website", "http://www.gnome.org/projects/gnome-games/",
+			       "wrap-license", TRUE,
+			       NULL);
 }
 
 void
@@ -455,6 +309,9 @@ button_press_event (GtkWidget *widget, GdkEventButton *event)
 {
 	guint x, y;
 
+	if (game_in_progress == 0)
+		return TRUE;
+
 	if (!network_allow ())
 		return TRUE;
 	
@@ -573,7 +430,13 @@ flip_pixmaps (gpointer data)
 	
 	for (i = 0; i < 8; i++)
 		for (j = 0; j < 8; j++) {
-			if (pixmaps[i][j] == 100) {
+			/* This first case only happens when undoing the "final flip". */
+			if ((pixmaps[i][j] == 101) && (board[i][j] != 0)) {
+				pixmaps[i][j] = board[i][j];
+				gui_draw_pixmap (pixmaps[i][j], i, j);
+				flipped_tiles = 1;				
+			} else if ((pixmaps[i][j] == 100) 
+				   || ((pixmaps[i][j] != 101) && (board[i][j] == 0))) {
 				pixmaps[i][j] = 101;
 				gui_draw_pixmap (0, i, j);
 				flipped_tiles = 1;
@@ -675,7 +538,6 @@ clear_board (void)
 			board[i][j] = 0;
 
 	memcpy (pixmaps, board, sizeof (gint8) * 8 * 8);
-	memcpy (game[0].board, board, sizeof (gint8) * 8 * 8);
 
 	bcount = 0;
 	wcount = 0;
@@ -688,8 +550,7 @@ init_new_game (void)
 {
 	clear_board ();
 	game_in_progress = 1;
-	move_count = 0;
-	whose_turn = BLACK_TURN;
+	move_count = 4;
 
 	undo_set_sensitive (FALSE);
 	
@@ -700,28 +561,15 @@ init_new_game (void)
 
 	bcount = 2;
 	wcount = 2;
+	init();
 
 	memcpy (pixmaps, board, sizeof (gint8) * 8 * 8);
-	memcpy (game[0].board, board, sizeof (gint8) * 8 * 8);
 
 	redraw_board ();
 
 	whose_turn = BLACK_TURN;
 	gui_message (_("Dark's move"));
 
-	games_clock_stop (GAMES_CLOCK (time_display));
-	games_clock_set_seconds (GAMES_CLOCK (time_display), 0);
-
-	if (black_computer_level ^ white_computer_level) {
-		if (! black_computer_level)
-			games_clock_start (GAMES_CLOCK (time_display));
-		gtk_widget_set_sensitive (time_display, TRUE);
-		timer_valid = 1;
-	} else {
-		gtk_widget_set_sensitive (time_display, FALSE);
-		timer_valid = 0;
-	}
-
 	check_computer_players ();
 }
 
@@ -739,8 +587,6 @@ create_window (void)
 
 	gnome_app_create_menus (GNOME_APP (window), mainmenu);
 
-	/* gtk_widget_push_colormap (gdk_rgb_get_cmap ()); */
-
 	drawing_area = gtk_drawing_area_new ();
 
 	gtk_widget_pop_colormap ();
@@ -757,6 +603,9 @@ create_window (void)
 			  G_CALLBACK (button_press_event), NULL);
 	gtk_widget_set_events (drawing_area,
 			       GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
+        /* We do our own double-buffering. */
+        gtk_widget_set_double_buffered (drawing_area, FALSE);
+
 	gtk_widget_show (drawing_area);
 
 	appbar = GNOME_APPBAR (gnome_appbar_new (FALSE, TRUE, FALSE));
@@ -784,13 +633,6 @@ create_window (void)
 	gtk_widget_show (white_score);
 
 	gtk_table_attach (GTK_TABLE (table), white_score, 5, 6, 0, 1, 0, 0, 3, 1);
-
-	time_display = games_clock_new ();
-	gtk_widget_set_sensitive (time_display, FALSE);
-	gtk_widget_show (time_display);
-
-	gtk_table_attach (GTK_TABLE (table), time_display, 7, 8, 0, 1, 0, 0, 3, 1);
-
 	undo_set_sensitive (FALSE);
 
 	gtk_widget_show (table);
@@ -918,7 +760,8 @@ int
 main (int argc, char **argv)
 {
 	GnomeClient *client;
-	struct timeval tv;
+	GnomeProgram *program;
+	GOptionContext *context;
 
 	gnome_score_init ("iagno");
 
@@ -926,15 +769,17 @@ main (int argc, char **argv)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	gettimeofday (&tv, NULL);
-	srand (tv.tv_usec);
+        context = g_option_context_new ("");
+        g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+
+	program = gnome_program_init ("iagno", VERSION,
+				      LIBGNOMEUI_MODULE,
+				      argc, argv,
+				      GNOME_PARAM_GOPTION_CONTEXT, context,
+				      GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
+
+	gtk_window_set_default_icon_name ("gnome-iagno");
 
-	gnome_program_init ("iagno", VERSION,
-			    LIBGNOMEUI_MODULE,
-			    argc, argv,
-			    GNOME_PARAM_POPT_TABLE, options,
-			    GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
-	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/iagno.png");
 	client= gnome_master_client ();
 
 	g_object_ref (G_OBJECT (client));
@@ -950,19 +795,23 @@ main (int argc, char **argv)
 
 	load_pixmaps ();
 
+	gtk_widget_show (window);
+
 	if (session_xpos >= 0 && session_ypos >= 0) {
 		gdk_window_move (window->window, session_xpos, session_ypos);
 	}
 
-	gtk_widget_show (window);
-
 	buffer_pixmap = gdk_pixmap_new (drawing_area->window,
 					BOARDWIDTH, BOARDHEIGHT, -1);
 
 	set_bg_color ();
 
+	init_new_game ();
+
 	gtk_main ();
 
+	g_object_unref (program);
+
 	return 0;
 }
 
